(* Attempt to build a host command with Dns module *)

(* type that represents 'host' configuration *)
type config =
{
   typ : int;
   hostserv : string list;
   wait : bool ref;  
   cls : int;
   recurs : bool ref;
   lst : bool ref
}
;;

let usage =
   "host: usage error (missing host argument)\n \
   Usage: host [-rw] [-t querytype] [-c class] host [server]\n \
	 -c class to look for non-Internet data \n \
	 -l to turn on 'list mode' \n \
	 -r to disable recursive processing \n \
         -t querytype to look for a specific type of information \n \
	 -w to wait forever until reply\n";;

(* This function will output answer the 'host' way *)
let rec find_n_print answ typ =
   match answ with
   | a::l -> 
      (if typ = 255 || a#get_type = typ
      then
      let quest = a#get_name in
      match a#get_rdata with
      | Dns.A str -> 
	 print_string (quest^" has address "^str^"\n")
      | Dns.AAAA str -> 
	 print_string (quest^" has address "^str^"\n")
      | Dns.CNAME str ->
	 print_string (quest^" is a nickname for "^str^"\n")
      | Dns.HINFO (os, cpu) (* ?! *)  ->
	 print_string (quest^" host information "^cpu^" "^os^"\n")
      | Dns.MX (prf, str) ->
	 print_string (quest^" mail is handled (pri="^(string_of_int prf)^
         ") by "^str^"\n")
      | Dns.NS str ->
	 print_string (quest^" name server "^str^"\n")
      | Dns.PTR name ->
	 print_string (quest^" domain name pointer "^name^"\n")
      | Dns.SOA (m, r, s, ref, ret, ex, min) ->
	 (print_string (quest^" start of authority  "^m^" "^r^"(\n");
	 Printf.printf "%32u ;serial (version)\n" s;
	 Printf.printf "%32u ;refresh period\n" ref;
	 Printf.printf "%32u ;retry refresh this often\n" ret;
	 Printf.printf "%32u ;expiration period\n" ex;
	 Printf.printf "%32u ;minimum TTL\n" min;
	 Printf.printf "%28s\n" ")")
      | Dns.TXT str ->
	 print_string (quest^"  descriptive text \""^str^"\"\n")
      | default ->
	 raise (Dns.Unknow_type ("Not supported type: "^
	    (Dnsprint.type_to_string (Dnsprint.rrtype_to_int default)))));
      find_n_print l typ
   | [] -> ();
;;

(* Let's parse the command line *)
 
let parse_args = 
   let default = ref
   {
      typ = 255;
      hostserv = [];
      wait = ref false;
      cls = 1; 
      recurs = ref false;
      lst = ref false;
   } 
   and set_type def s =
      def := { !def with
	 typ = Dnsprint.string_to_type (String.uppercase s)}
   and set_cls def x =
      def := { !def with cls = x}
   and anon def s =
      def := { !def with hostserv = !def.hostserv@[s]}
   in
   let lst = 
      [("-c", Arg.Int (set_cls default), "");
      ("-l", Arg.Set !default.lst, "");
      ("-r", Arg.Set !default.recurs, "");
      ("-t", Arg.String (set_type default), "");
      ("-w", Arg.Set !default.wait, "")] 
   in
   Arg.parse lst (anon default) usage;
   !default
;;


let rec classic set_server host conf search =
   let dns = new Dns.dns_query set_server in
   let pkt = dns#get_query in
   (* -r : Do a recursive request ?  *)
   if !(conf.recurs)
   then
      pkt#set_rd false
   else
      pkt#set_flags 256;

   (* Build the query using -c and -t *)
   if !(conf.lst)
   then
      dns#add_query host 252 conf.cls
   else
      dns#add_query host conf.typ conf.cls;

   (* wait forever *)
   if !(conf.wait)
   then
      dns#set_timeout (-1.);

   (* do the job *)
   dns#action;
   (* finally print the answer *)
   match dns#get_answer#get_rcode with
   | 0 ->
      find_n_print dns#get_answer#get_answers 255; (* ANY *)
   | 3 | 5 -> 
      if search != []
      then
         classic set_server (host^"."^(List.hd search)) conf (List.tl search)
      else
	 print_string "Host not found.\n"
   | other -> 
      print_string ("TODO: error number "^(string_of_int other)^".");
;;


let axfr set_server host =
   let axfr = new Dns.dns_axfr set_server in
   axfr#ls host;
   axfr#action;
   let lst = axfr#get_lst in
   match (List.hd lst)#get_rcode with
   | 0 -> 
      List.iter (fun x ->
	 find_n_print x#get_answers 255) lst; 
   | 5 -> 
      print_string "Server failed: Query refused\n"
   | other -> 
      print_string ("TODO: error number "^(string_of_int other)^".")
;;

try
let conf = parse_args in
let host = List.hd conf.hostserv
and servers, domain, search = Tools.parse_resolv in
let set_server =
   let temp =
      if List.length conf.hostserv >= 2
      then
	 begin
	 let srv = List.nth conf.hostserv 1 in
	 print_string ("Using domain server "^srv^"\n");
	 srv
      end
      else
	 List.hd servers
   in
   Tools.getaddr temp
in

(* Let's use command line arguments *)

if !(conf.lst)
then
   axfr set_server host
else
   classic set_server host conf search
with
| Failure("hd") -> (print_string usage; exit 1);
| Dns.Unknow_type str -> print_string ("Caught Exception: unknow type "^str^"\n")
;;

