(** Set of usefull tools.
 *)

(** {i hexa_of_int integer size} will output a string representation of 
    {i integer} in hexa on {i size} bytes.
    example:
    # hexa_of_int 65 4;;
    - : string = "\000\000\000A"
    *)
let hexa_of_int nbr size=
    let buf = Buffer.create 30 in
    let rec temp nbre indice =
    	let zero = int_of_float (256. ** indice) in
	let div = if zero == 0 then 1 else zero in
 	let res = nbre/div in
 	if res == 0 && indice > 0. || nbre >= 256
	then 
 	    begin
 	    Buffer.add_char buf (Char.unsafe_chr res);
 	    temp (nbre-(res*256)) (indice-.1.)
 	    end
 	else
	    Buffer.add_char buf (Char.unsafe_chr nbre)
    in
 	temp nbr ((float_of_int size) -. 1.);
 	Buffer.contents buf
;;

(** {b is_print chr} returns {b true} if {i chr} is a printable character 
   (like {b isprint ()} in C) {b false} if not. *)
let isprint chr =
   let ascii = Char.code chr in
   if ascii >= 32 && ascii <= 126
   then
      true
   else
      false
;;

(** {b dump_steam stm} will output the stream in hexa and in ascii. It's very
   usefull for network debugging. *)
let dump_stream stm =
   let hexa = ref [] 
   and ascii = ref [] in
   try
   let i = ref 0 in
   print_string "\n\n";
   while true
   do
      let chr = Stream.next stm in 
      hexa := !hexa@[Printf.sprintf "%.2x " (Char.code chr)];
      let chascii = if isprint chr then chr else '.' in
      ascii := !ascii@[Printf.sprintf " %2c " chascii];
      i := !i + 1;
      if (!i-(!i/8)*8) = 0
      then
	 begin
	 List.iter print_string !hexa;
	 print_string "  ";
	 List.iter print_string !ascii;
	 print_char '\n';
	 hexa := [];
	 ascii := []
	 end
   done
   with
   | Stream.Failure ->
      begin
      let i = ref 0 in
      List.iter (fun x -> i := !i +1; print_string x) !hexa;
      let complete = 8 - !i in
      if complete >= 0
      then 
   	 for x=0 to complete-1 do print_string "   " done;
      print_string "  ";
      List.iter print_string !ascii;
      print_string "\n\n";
      end
;;

(** {b parse_resolv ()} parse /etc/resolv.conf and return a tuple 
   that represents it contents.
   @return list of servers
   @return domain
   @return list of search domains
 *)
let parse_resolv =
   let file = open_in "/etc/resolv.conf"
   and server = ref []
   and search = ref []
   and domain = ref "" in
   let rec loop () =  
      let line = input_line file in
      if (Netstring_str.string_match (Netstring_str.regexp "^#") line 0) = None
      then
         (match Netstring_str.split (Netstring_str.regexp " ") line with
         | ["nameserver"; dns] -> server := !server@[dns]
         | ["domain"; dom] -> domain := dom
	 | "search"::lst -> search := !search@lst
	 | _ -> print_string "Error while parsing /etc/resolv.conf\n"; exit 0);
         loop ()
   in
   try loop () with 
      | End_of_file -> let ret = (!server, !domain, !search) in close_in file; ret
;;

(** {b daemonize "test.log" ping_server} will put ping_server to background and
   log {b stdout} and {b stderr} to "test.log". *)
let daemonize logfile funct = 
   match Unix.fork () with 
   | 0 -> (match Unix.setsid () with
	  | -1 -> (print_string "Exiting: setsid() error\n"; exit 0)
	  | _ ->
	       let fd = Unix.openfile "/dev/null" [Unix.O_RDONLY] 0 in
	       let log = Unix.openfile logfile [Unix.O_WRONLY;Unix.O_APPEND;Unix.O_CREAT] 0600 in
	       Unix.dup2 fd (Unix.descr_of_in_channel stdin);
	       Unix.close fd;
	       Unix.dup2 log (Unix.descr_of_out_channel stdout);
	       funct ()
	       )
   | pid -> exit 0
;;

exception Toosmall of int;;

(** This function will extract a new stream of {i size} elements from {i stm}.
    @raise Toosmall when {i stm} is too small *)
let extract_stream stm size =
   let buf = Buffer.create size
   and i = ref 0
   and loop = ref true in
   try
   while !i < size
   do
      Buffer.add_char buf (Stream.next stm);
      i := !i +1;
   done;
   Stream.of_string (Buffer.contents buf)
   with
   | Stream.Failure -> raise (Toosmall !i)
;;

(** This function will convert a {b char Stream.t} to a {b string} *)
let stream_to_string stm =
   let buf = Buffer.create 5 in
   try
   while true
   do
      Buffer.add_char buf (Stream.next stm)
   done;
   ""
   with
   | Stream.Failure -> Buffer.contents buf
;;

(** This function will change the uid and gid of a process 
     own by root to the uid and id given by {i usr}. *)
let downprivs usr =
  let getid user=
   try
   let pwu = Unix.getpwuid (int_of_string user) in
   (pwu.Unix.pw_uid, pwu.Unix.pw_gid)
   with
   | Failure "int_of_string" | Not_found -> 
      (try
      let pwu = Unix.getpwnam user in
      (pwu.Unix.pw_uid, pwu.Unix.pw_gid)
      with 
      | Not_found ->
	 print_string ("No such user: "^user^"\n"); exit 0
      )
   in
   let uid, gid = getid usr in
   try
   Unix.setgid gid;
   Unix.setuid uid
   with
   | Unix.Unix_error(_, "setgid", _)
   | Unix.Unix_error(_, "setuid", _) ->
      (print_string "downprivs: Error (are you root ?!)\n"; exit 0)
;;

(** This function gives the IP address of {i str}. *)
let getaddr str =
   try 
   ignore (Unix.inet_addr_of_string str);
   str (* IP address *)
   with
   | Failure _ ->
      try
      Unix.string_of_inet_addr (Unix.gethostbyname str).Unix.h_addr_list.(0)
      with
      | Not_found -> print_string ("Error looking up : "^str^"\n");
		     exit 0
;;
